PROGRAM_NAME='tokenizer'
(***********************************************************)
(*  FILE CREATED ON: 07/27/2005  AT: 13:43:23              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 10/31/2008  AT: 18:55:08        *)
(***********************************************************)
(*  FILE REVISION: 2.08                                    *)
(*  REVISION DATE: 10/31/2008  AT: 18:55:08                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed tweaks                                         *)
(*    * Reduce FOR and WHILE loop control expressions to   *)
(*      only the variable name where possible.             *)
(*    * Avoid use of LENGTH_ARRAY or LENGTH_STRING() in    *)
(*      FOR or WHILE loop control expressions.             *)
(*    For example:                                         *)
(*    Bad  -   FOR (i = 1; i <= LENGTH_STRING(str); i++)   *)
(*    Good -   len = LENGTH_STRING(str)                    *)
(*             FOR (i = 1; i <= len; i++)                  *)
(*    Better - FOR (i = LENGTH_STRING(str); i > 0; i--)    *)
(*    Best -   FOR (i = LENGTH_STRING(str); i; i--)        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.07                                    *)
(*  REVISION DATE: 10/06/2008  AT: 07:26:34                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.06                                *)
(*  REVISION DATE: 08/26/2008  AT: 08:31:14                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed deprecated function slStringTokenizer().     *)
(*  - String-to-be-processed parameters of subroutines     *)
(*    were set to 15999 to workaround a feature that       *)
(*    truncates string expressions to 2048 bytes if passed *)
(*    in via an unbounded array.                           *)
(*    With the parameter size set to 15999 they are not    *)
(*    truncated below this "normal" NetLinx interpreter    *)
(*    limit.                                               *)
(*    String arrays, for example the output of FILE_READ() *)
(*    or GET_BUFFER_STRING(), will still pass through      *)
(*    unaffected at 16000 bytes or greater.                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.05                                *)
(*  REVISION DATE: 08/25/2008  AT: 12:27:42                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Changed name from 'strings.axi' to 'tokenizer.axi'.  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.04                                *)
(*  REVISION DATE: 04/10/2007  AT: 18:24:12                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added StringTokenizerEsc(). Data between two escape  *)
(*    characters will not be parsed for delim. Only        *)
(*    supports single-byte delims.                         *)
(***********************************************************)
(*  FILE REVISION: 2.03                                    *)
(*  REVISION DATE: 03/17/2007  AT: 14:47:00                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - StringTokenizer returns the number tokens found as   *)
(*    integer.                                             *)
(***********************************************************)
(*  FILE REVISION: Rev 2.02                                *)
(*  REVISION DATE: 03/12/2007  AT: 13:31:25                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - StringTokenizer now returns results to a passed-in   *)
(*    array of strings. This will allow more flexibility   *)
(*    as the caller can then convert the data by context.  *)
(*  - To simplify parameters, the function now expects any *)
(*    message header to be stripped off before calling the *)
(*    function.                                            *)
(*  - Old StringTokenizer function has been renamed to     *)
(*    slStringTokenizer.                                   *)
(***********************************************************)
(*  FILE REVISION: Rev 2.01                                *)
(*  REVISION DATE: 03/09/2007  AT: 17:12:30                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Improvements to StringTokenizer:                     *)
(*    * No longer requires a delimiter between msgHeader   *)
(*      and arg1.                                          *)
(*    * delim may be more than 1 character long.           *)
(*    * returns slong, not integer.                        *)
(***********************************************************)
(*!!FILE REVISION: 2.00 GLM                                *)
(*  REVISION DATE: 07/27/2005                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(* string functions and calls                              *)
/* 1    function StringTokenizer (args[], delim[], results[][])               */
/* 2    function StringTokenizerEsc (args[], delim, escape, results[][])      */
(***********************************************************)
#IF_NOT_DEFINED __tokenizer_lib__
#DEFINE __tokenizer_lib__

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE 

CHAR __TOKENIZER_LIB_NAME__[16]     = '   tokenizer.axi';
CHAR __TOKENIZER_LIB_VERSION__[]    = '2.08';

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

/***********************************/
/* DEFINE_FUNCTION StringTokenizer */
/***********************************/
/* 
 * this function requires the data be of 
 * the form "<arg1><delim>...<delim><argX>"
 *
 * purpose:
 *	non-destructively parse delimited string values from a larger character string.
 *
 * parameters:
 *	args[]      - raw string with delimited data. 
 *	delim[]     - delimiter between data. 
 *	result[][]  - results are stored in this array. 
 * 
 * returns:
 *	number of tokens found
 *
 */
DEFINE_FUNCTION INTEGER StringTokenizer(CHAR args[15999], CHAR delim[], CHAR sArgArray[][])
{
  STACK_VAR INTEGER nStart;
  STACK_VAR INTEGER nNbrOfValues;

									// Get the start position past the header
  nStart = 1;			            // make sure start position is non-zero

									// if there is more than one value, there will be at least
									// one delimiter.
  IF (FIND_STRING(args,"delim",nStart) > nStart)
  {
	nNbrOfValues = 1;				// increment value counter/index from 0 to 1
									// store the value
	sArgArray[nNbrOfValues] = MID_STRING(args,
										nStart,
										FIND_STRING(args,"DELIM",nStart) - nStart);

									// find the next delimiter, set nStart to that value
	nStart = FIND_STRING(args,"DELIM",nStart);

  									// while there's a delimiter at nStart
	WHILE (FIND_STRING(args,"delim",nStart))
	{
									// set nStart to beginning of data string
	  nStart = nStart + LENGTH_ARRAY(delim);
	
	  nNbrOfValues++;				// increment value counter/index
	
									// if this is not the last value
	  IF (FIND_STRING(args,"DELIM",nStart))
	  {
									// store the value
		sArgArray[nNbrOfValues] = MID_STRING(args,
											nStart,
											FIND_STRING(args,"DELIM",nStart) - nStart);
	
									// find the next delimiter, set nStart to that value
		nStart = FIND_STRING(args,"DELIM",nStart);
	  }
	  ELSE                          // if this is the last value
	  {
									// store the value
		sArgArray[nNbrOfValues] = RIGHT_STRING(args,LENGTH_ARRAY(args) - (nStart - 1))
	  }
	}
  }
									// If there is only one value and no delim
  ELSE IF (FIND_STRING(args,"delim",nStart) = 0 && (LENGTH_ARRAY(args) >= nStart))
  {
	nNbrOfValues = 1;				// increment value counter/index from 0 to 1

									// store the value
	sArgArray[nNbrOfValues] = RIGHT_STRING(args,LENGTH_ARRAY(args) - (nStart - 1))
  }
  
  SET_LENGTH_ARRAY(sArgArray,nNbrOfValues);  
  RETURN nNbrOfValues;
}

/**************************************/
/* DEFINE_FUNCTION StringTokenizerEsc */
/**************************************/
/* 
 * this function requires the data be of 
 * the form "<arg1><delim>...<delim><argX>"
 *
 * purpose:
 *	non-destructively parse delimited string values from a larger character string.
 *
 * parameters:
 *	args[]      - raw string with delimited data. 
 *	delim		- single-byte delimiter between data.
 *  escape      - single-byte escape character. Double-quote is typically used.
 *                data inside escaped area is not parsed for delim's. 
 *	result[][]  - results are stored in this array. 
 * 
 * returns:
 *	number of tokens found
 *
 */
DEFINE_FUNCTION INTEGER StringTokenizerEsc(CHAR args[15999], CHAR delim, CHAR escape, CHAR sArgArray[][])
{
  STACK_VAR INTEGER i;				// outer loop counter
  STACK_VAR INTEGER byteCounter;
  STACK_VAR INTEGER nTokenCount;
  STACK_VAR INTEGER bEscaped;
  STACK_VAR INTEGER len;

  IF (LENGTH_ARRAY(args))
  {
	nTokenCount = 1
	byteCounter = 0
	len = LENGTH_ARRAY(args)

	FOR (i = 1; i <= len; i++)
	{
	  SELECT
	  {
		ACTIVE (bEscaped == TRUE):  // if inside an escaped area, don't search for delim's...
		{
		  IF (args[i] = escape)			// if end of escape is found...
			bEscaped = FALSE        	// ...clear the escaped flag...

		  byteCounter++             	// ...add data to sArgArray[nTokenCount]
		  sArgArray[nTokenCount][byteCounter] = args[i]
		}
		ACTIVE (args[i] == escape): // else if a start of escape area is found...
		{
		  bEscaped = TRUE           	// ...set escaped flag...

		  byteCounter++             	// ...add data to sArgArray[nTokenCount]
		  sArgArray[nTokenCount][byteCounter] = args[i]
		}
		ACTIVE (args[i] == delim):	// else if a new delimiter is found...
		{
										// ...set the byte count of token...
		  SET_LENGTH_ARRAY(sArgArray[nTokenCount],byteCounter)

		  nTokenCount++					// ...increment token count...
		  byteCounter = 0               // ...and reset byteCounter
		}
		ACTIVE (1):					// else...
		{
		  byteCounter++             	// ...add data to sArgArray[nTokenCount]
		  sArgArray[nTokenCount][byteCounter] = args[i]
		}
	  }
	}
									// set the length of final token
	SET_LENGTH_ARRAY(sArgArray[nTokenCount],byteCounter)
  }
  
  SET_LENGTH_ARRAY(sArgArray,nTokenCount);
  RETURN nTokenCount;
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
  ONLINE:
  {
	IF (__display_lib_version__ == TRUE)
	{
	  wait 50                       // WAIT FOR DEVICE HOLDOFF
	  {
		  // WHAT VERSION? 
		SEND_STRING 0:0:0,"'Using lib ',__TOKENIZER_LIB_NAME__,
								  ', v',__TOKENIZER_LIB_VERSION__";
	  }
	}
  }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
